// Utils.cpp: implementation of the CUtils class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "utils.h"
#include "main.h"

#ifndef WS_EX_LAYERED
	#define LWA_ALPHA 2
	#define WS_EX_LAYERED 0x80000
#endif
typedef BOOL (WINAPI *PFN_SETLAYEREDWINDOWATTRIBUTES)(HWND, COLORREF, BYTE, DWORD);

///////////////////////////////////////////////////////////////////////////////
//
// Retrieve handle of the desktop window
//
HWND utils::GetLitestepDesktop()
{
	HWND hWnd = FindWindow("DesktopBackgroundClass", NULL);
	return hWnd ? hWnd : GetDesktopWindow();

}


///////////////////////////////////////////////////////////////////////////////
//
// CreateLink
//    uses the Shell's IShellLink and IPersistFile interfaces 
//    to create and store a shortcut to the specified object. 
// Returns the result of calling the member functions of the interfaces. 
// Given parameters:
//   lpszPathObj  - address of a buffer containing the path of the object. 
//   lpszPathLink - address of a buffer containing the path where the 
//                  Shell link is to be stored. 
//   lpszDesc     - address of a buffer containing the description of the 
//                  Shell link. 
 
HRESULT utils::CreateLink(LPCSTR lpszPathObj, LPCSTR lpszPathLink, LPCSTR lpszDesc)
{ 
	HRESULT hres; 
	IShellLink* psl; 

	hres = CoCreateInstance(CLSID_ShellLink, NULL,
		CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *) &psl); 
	if (SUCCEEDED(hres))
	{
		IPersistFile* ppf; 
		psl->SetPath(lpszPathObj); 
		psl->SetDescription(lpszDesc);
 
		hres = psl->QueryInterface(IID_IPersistFile, (LPVOID*)&ppf); 
		if (SUCCEEDED(hres))
		{
			WCHAR wsz[MAX_PATH]; 
            MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1, wsz, MAX_PATH);  
			hres = ppf->Save(wsz, TRUE); 
			ppf->Release(); 
		}
		psl->Release(); 
	}
	return hres; 
} 


///////////////////////////////////////////////////////////////////////////////////////
//
// Check if point is in rectangle...
//
bool utils::IsPointInRect(int xPos, int yPos, const RECT *rect)
{
	return (rect->left <= xPos && xPos <= rect->right && rect->top <= yPos && yPos <= rect->bottom);
}


///////////////////////////////////////////////////////////////////////////////
//
// Error Box
//
void utils::ErrorBox(const char *szMessage)
{
	MessageBox(NULL, szMessage, "LiteStep :: Clickonic", MB_OK | MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND);
	//LSLogPrintf(LOG_ERROR, g_szAppName, szMessage);
}

void utils::ErrorBoxPrint(const char *szFormat, ...)
{
	// Format the message
	char szMessage[MAX_LINE_LENGTH];
	va_list argList;

	va_start(argList, szFormat);
	StringCchVPrintf(szMessage, MAX_LINE_LENGTH, szFormat, argList);
	va_end(argList);

	ErrorBox(szMessage);
}

/*
void utils::log(const char *szFormat, ...)
{
	// Format the message
	char szMessage[MAX_LINE_LENGTH];
	va_list argList;

	va_start(argList, szFormat);
	StringCchVPrintf(szMessage, MAX_LINE_LENGTH, szFormat, argList);
	va_end(argList);

	LSLog(LOG_DEBUG, g_szAppName, szMessage);
}*/

void utils::CopyString(char **pszString, const char *szValue)
{
	int nLen = strlen(szValue) + 1;
	*pszString = (char*)HeapAlloc(GetProcessHeap(), 0, nLen);
	StringCchCopy(*pszString, nLen, szValue);
}


//
// Control amount of icons groups in the registry
//
void GarbageCollector()
{
	
}

COLORREF utils::String2Color(const char *szColor)
{
	COLORREF crTextColor = strtol(szColor, NULL, 16);
	// convert from BGR to RGB
	return RGB(GetBValue(crTextColor), GetGValue(crTextColor), GetRValue(crTextColor));
}

bool utils::String2Bool(const char *szBool)
{
	return (lstrcmpi(szBool, "off") && lstrcmpi(szBool, "false") && lstrcmpi(szBool, "no"));
}

bool utils::File_Exists (char* szFilePath) {
	return GetFileAttributes(szFilePath) != INVALID_FILE_ATTRIBUTES;
}